/*
   grammar.y	-- a parser of Tiny C
*/

%{
#include <stdio.h>
%}

/* yacc produces yyparse() from the following grammar. */

%token Identifier
%token Number

%token '=' '+' '-' '*' '/' ',' ';' '(' ')' '{' '}' '[' ']'

%token INT
%token RETURN
%token IF
%token BADTOKEN

%%

program
    : function
    | program function
    | error '}'		/* error recovery: ignore tokens until '}' */
			/*		   is encountered.	   */

function
    : typename Identifier '(' formal.arguments ')' function.body

typename
    : INT

formal.arguments
    : /* empty */
    | formal.argument.list

formal.argument.list
    : formal.argument
    | formal.argument.list ',' formal.argument

formal.argument
    : typename Identifier

function.body
    : '{' statements '}'

statements
    : statement
    | statements statement

statement
    : typename Identifier ';'		/* variable declaration */
    | RETURN expression ';'
    | Identifier '=' expression ';'	/* assignment */
    | expression ';'
    | ';'				/* null statement */

/* The last two rules make shift/reduce conflicts. */
expression
    : Identifier				/* variable name */
    | Number					/* integer */
    | Identifier '(' actual.arguments ')'	/* function call */
    | expression '+' expression
    | expression '-' expression

actual.arguments
    : expression
    | actual.arguments ',' expression

%%

yyerror(msg)
    char* msg;
{
    extern int yynerrs;

    ++yynerrs;
    puts(msg);
}

main()
{
    extern int yynerrs;
    int result, nerrors;

#ifdef YYDEBUG
    yydebug = !0;
#endif
    result = yyparse();
    nerrors = yynerrs;
    printf("result %d (%d errors.)\n", result, nerrors);
}
